/* ========================================================================
 * JCommon : a free general purpose class library for the Java(tm) platform
 * ========================================================================
 *
 * (C) Copyright 2000-2004, by Object Refinery Limited and Contributors.
 * 
 * Project Info:  http://www.jfree.org/jcommon/index.html
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc. 
 * in the United States and other countries.]
 * 
 * ------------------
 * TextUtilities.java
 * ------------------
 * (C) Copyright 2004, by Object Refinery Limited and Contributors.
 *
 * Original Author:  David Gilbert (for Object Refinery Limited);
 * Contributor(s):   -;
 *
 * $Id: TextUtilities.java,v 1.10 2004/05/21 17:32:22 mungady Exp $
 *
 * Changes
 * -------
 * 07-Jan-2004 : Version 1 (DG);
 * 24-Mar-2004 : Added 'paint' argument to createTextBlock() method (DG);
 * 07-Apr-2004 : Added getTextBounds() method and useFontMetricsGetStringBounds flag (DG);
 * 08-Apr-2004 : Changed word break iterator to line break iterator in the createTextBlock()
 *               method - see bug report 926074 (DG);
 *
 */

package org.jfree.text;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.geom.Rectangle2D;
import java.text.BreakIterator;

import org.jfree.util.LogContext;
import org.jfree.util.Log;

/**
 * Some utility methods for working with text.
 */
public abstract class TextUtilities {

    /** Access to logging facilities. */
    protected static final LogContext logger = Log.createContext(TextUtilities.class);
    
    /**
     * Creates a new text block from the given string.
     * 
     * @param text  the text.
     * @param font  the font.
     * @param paint  the paint.
     * @param maxWidth  the maximum width for each line.
     * @param measurer  the text measurer.
     * 
     * @return A text block.
     */
    public static TextBlock createTextBlock(final String text,
                                            final Font font,
                                            final Paint paint,
                                            final float maxWidth,
                                            final TextMeasurer measurer) {
        return createTextBlock(text, font, paint, maxWidth, Integer.MAX_VALUE, measurer);
    }
    
    /**
     * Creates a new text block from the given string.
     * 
     * @param text  the text.
     * @param font  the font.
     * @param paint  the paint.
     * @param maxWidth  the maximum width for each line.
     * @param maxLines  the maximum number of lines.
     * @param measurer  the text measurer.
     * 
     * @return A text block.
     */
    public static TextBlock createTextBlock(final String text,
                                            final Font font,
                                            final Paint paint,
                                            final float maxWidth,
                                            final int maxLines,
                                            final TextMeasurer measurer) {
        final TextBlock result = new TextBlock();
        final BreakIterator iterator = BreakIterator.getLineInstance();
        iterator.setText(text);
        int current = 0;
        int lines = 0;
        final int length = text.length();
        while (current < length && lines < maxLines) {
            final int next = nextLineBreak(text, current, maxWidth, iterator, measurer);
            if (next == BreakIterator.DONE) {
                result.addLine(text.substring(current), font, paint); 
                return result;
            }
            result.addLine(text.substring(current, next), font, paint);
            lines++;
            current = next;
        }
        return result;
    }
    
    /**
     * Returns the character index of the next line break.
     * 
     * @param text  the text.
     * @param start  the start index.
     * @param width  the end index.
     * @param iterator  the word break iterator.
     * @param measurer  the text measurer.
     * 
     * @return The index of the next line break.
     */
    private static int nextLineBreak(final String text, final int start, final float width,
                                     final BreakIterator iterator, final TextMeasurer measurer) {
        // this method is (loosely) based on code in JFreeReport's TextParagraph class
        int current = start;
        int end;
        float x = 0.0f;
        boolean firstWord = true;
        while (((end = iterator.next()) != BreakIterator.DONE)) {
            x += measurer.getStringWidth(text, current, end);  
            if (x > width) {
                if (firstWord) {
                    while (measurer.getStringWidth(text, start, end) > width) {
                        end--;
                    }
                    //iterator.setPosition(end);
                    return end;
                }
                else {
                    end = iterator.previous();
                    return end;
                }
            }
            // we found at least one word that fits ...
            firstWord = false;
            current = end;
        }
        return BreakIterator.DONE;
    }
    
    /**
     * Returns the bounds for the specified text.
     * 
     * @param text  the text (<code>null</code> permitted).
     * @param g2  the graphics context (not <code>null</code>).
     * @param fm  the font metrics (not <code>null</code>).
     * 
     * @return The text bounds (possibly <code>null</code>).
     */
    public static Rectangle2D getTextBounds(final String text, 
                                            final Graphics2D g2, final FontMetrics fm) {
        Rectangle2D bounds = null;
        if (TextUtilities.useFontMetricsGetStringBounds) {
            bounds = fm.getStringBounds(text, g2);
        }
        else {
            final double width = fm.stringWidth(text);
            final double height = fm.getHeight();
            if (logger.isDebugEnabled()) {
                logger.debug("Height = " + height);   
            }
            bounds = new Rectangle2D.Double(0.0, -fm.getAscent(), width, height);
        }
        return bounds;
    }
    
    /** 
     * A flag that controls whether the FontMetrics.getStringBounds() method is used or
     * a workaround is applied.
     */
    private static boolean useFontMetricsGetStringBounds = false;

    /**
     * Returns the flag that controls whether the FontMetrics.getStringBounds() method is used or
     * not.  If you are having trouble with label alignment or positioning, try changing the value
     * of this flag.
     * 
     * @return A boolean.
     */
    public static boolean getUseFontMetricsGetStringBounds() {
        return useFontMetricsGetStringBounds;    
    }
    
    /**
     * Sets the flag that controls whether the FontMetrics.getStringBounds() method is used or
     * not.  If you are having trouble with label alignment or positioning, try changing the value
     * of this flag.
     * 
     * @param use  the flag.
     */
    public static void setUseFontMetricsGetStringBounds(boolean use) {
        useFontMetricsGetStringBounds = use;   
    }
    
}
